% Try to compute the stationary solution of the
% frequency comb model with Newton's method

% Strategy: The complex solution u is decomposed in its real and
% imaginary part, i.e. u=v+iw. Thus, we seek a zero of the function
%
% F(v,w) = [-v + zeta*w - d*A*w - (v^2+w^2)*w + f
%           -w - zeta*v + d*A*v + (v^2+w^2)*v  ]
%
% The original problem is posed on (0,2pi) with periodic boundary conditions.
% In order to avoid ambiguity due to periodicity, we consider the problem
% on (0,pi) with homogeneous Neumann boundary conditions.
%
% For the spectral differentiation, the solution is represented by a cosine
% series. This means that only the second (but not the first or third)
% spatial derivative can be computed.


clear all hidden; clc; close all hidden; drawnow

fprintf('\n\n\n')
fprintf('-------------------------------- \n')
fprintf('Compute comb with Newtons method \n')
fprintf('-------------------------------- \n\n\n')

% Promt uses on the directory 
autodir = input('Please enter the sub-directory \nof the current directory \nwith the auto data [e.g. mydir] ','s');
% Configuration -----------------------------------------------------------

kmax=1001; % We consider cosine modes 0 to kmax.
N=100; % Maximal number of Newton steps
tol=3e-10; % Newton method stops when residual is smaller than tol.

% My stopping criterion is actually NOT GOOD. I have to change this when I
% have more time.


% Consequences ------------------------------------------------------------

temp=kmax+1;
if abs(temp/2-round(temp/2))>0.1
    fprintf('Warning: Parameter kmax must be odd. I will use kmax = %g insted of kmax = %g \n\n',kmax+1,kmax);
    kmax=kmax+1;
end
kvec=(0:kmax)';
h=pi/kmax;
xvec=kvec*h;


% Import and adapt AUTO's data --------------------------------------------

%path1 = '../f=1.6,d=0.1/';
path1 = strcat('../',autodir,'/');
cd(path1) 
% Format of parfile: ! Format branch-no., solution_no., zeta, f, d
datei1 = 'parfile';
fid=fopen(datei1,'r');
[p1,p2,zeta_list,f_list,d_list] = textread(datei1, '%s %s %f %f %f');
nn=length(p1);
for i=1:nn    
zeta = zeta_list(i);
f = f_list(i);
d = d_list(i);
datei = char(strcat('sol', p2(i), '_branch', p1(i)));
fid=fopen(datei,'r');
[col1,col2,col3,col4,col5] = textread(datei, '%f %f %f %f %f');
path2 ='../FC-Stationary-Newton';
cd(path2)
% autodata01
% autodata02
% autodata03_random_data

% Replace "autodata..." by the name of the m-file which contains the
% parameters f, d, zeta and the AUTO data


% Adapt AUTO's data by spline interpolation -------------------------------

%a1=interp1(data(:,1),data(:,2),(0:kmax)'/kmax,'spline');
a1=interp1(col1,col2,(0:kmax)'/kmax,'spline');
%a2=interp1(data(:,1),data(:,4),(0:kmax)'/kmax,'spline');
a2=interp1(col1,col4,(0:kmax)'/kmax,'spline');
ahat0=a1+1i*a2;
v0=real(ahat0);
w0=imag(ahat0);
c=[zeta,d,f];

figure(10)

subplot(2,1,1)
%plot(data(:,1)*pi,data(:,2),'r.',xvec,a1,'b')
plot(col1*pi,col2,'r.',xvec,a1,'b')
ylabel('real part')
title('Initial data')
legend('AUTO data','Interpolation','Location','NorthEastOutside')

subplot(2,1,2)
%plot(data(:,1)*pi,data(:,4),'r.',xvec,a2,'b')
plot(col1*pi,col4,'r.',xvec,a2,'b')
ylabel('imaginary part')
legend('AUTO data','Interpolation','Location','NorthEastOutside')
xlabel('x')

drawnow

temp=get(gcf,'Position'); set(gcf,'Position',temp-[100 0 0 0])

% Cosine transform and spectral differentiation ---------------------------

% Instead of applying FFT to the function, the spectral differentiation 
% matrix is explicitly assembled. The spectral differentiation corresponds 
% to a multiplication of the vector with the matrix A.

% Of course, this is less efficient than FFT, but the additional
% difficulties (cosine transform instead of "full" FFT, imaginary numbers
% and so on) are avoided.

Tinv=zeros(kmax+1,kmax+1);
for k=0:kmax
    temp=kvec'*k*h;
    Tinv(k+1,:)=cos(temp);
end
T=inv(Tinv);
A=-Tinv*diag(kvec.^2)*T;

if 0
    % Test spectral differentiation
    clc;
    j=floor(rand*kmax)
    u=cos(j*xvec);
    Lu=-(j^2)*u;
    norm(Lu-A*u)
end


% Newton's method ---------------------------------------------------------

v=v0;
w=w0;
normF=2*tol;
n=0;

% My stopping criterion is actually NOT GOOD. I have to change this when I
% have more time.

while normF>tol & n<N & normF<1e10
    n=n+1;
    
    F1 = -v + zeta*w - d*A*w - (v.^2+w.^2).*w + f;
    F2 = -w - zeta*v + d*A*v + (v.^2+w.^2).*v;    
    
    F=[F1;F2];
    normF(n)=norm(F,inf);
    
    fprintf('   Step %g: residual %g \n',n,normF(n))
    
    DF=[-eye(kmax+1,kmax+1)-diag(2*v.*w), ...
        zeta*eye(kmax+1,kmax+1)-d*A-diag(v.^2+3*w.^2); ...
        -zeta*eye(kmax+1,kmax+1)+d*A+diag(3*v.^2+w.^2), ...
        -eye(kmax+1,kmax+1)+diag(2*v.*w)];  
    
    x=-DF\F;
    x=reshape(x,kmax+1,2);
    v=v+x(:,1);
    w=w+x(:,2);
    
    figure(20)
    subplot(2,1,1)
    plot(xvec,v0,'k',xvec,v,'b')
    title(['Stationary solution on [0,2pi] after step ',num2str(n)])
    ylabel('Real part')
    
    subplot(2,1,2)
    plot(xvec,w0,'k',xvec,w,'b')
    ylabel('Imaginary part')
    xlabel('x')
    
    % keyboard
end
fprintf('\n')
temp=norm(v0+1i*w0 - (v+1i*w),inf);
fprintf('Maximal difference between initial data and final result: %g \n\n',temp)
if normF>tol
  fprintf('No convergence! I have stopped the iteration because the residual is too big.\n')
  fprintf('Bad luck ... try again with different parameters and/or initial data.\n\n\n')
  error('  ')
end  
    
    

% Reconstruct solution on [0,2pi] -----------------------------------------

v=[v(1:kmax);v((kmax+1):-1:2)];
w=[w(1:kmax);w((kmax+1):-1:2)];
ahat=v+1i*w;

% The problem is that the length of u is always even due to the symmetric
% continuation, but myfft can only applied to a vector with odd length.
% In order to fix this, we consider every second point of u. This yields a
% vector of length kvec, and if kvec is odd, myfft can be applied.

ahat=ahat(1:2:end);
a=myfft(ahat);

hh=2*pi/(kmax+1);
xxvec=(0:kmax-1)*hh;


ms=3; % Marker Size
lw=2; % Line Width
fs=15; % FontSize'


figure(50)
cd(path1)

K=(kmax-1)/2;
% semilogy(-K:K,abs(a))
%temp=stem(-K:K,log(abs(a)),'fill','MarkerSize',ms,'LineWidth',lw);
temp=plot(-K:K,log(abs(a)),'.','LineWidth',lw);
%baseline = get(temp,'Baseline');
%set(baseline,'BaseValue',-40)
xlim([-200, 200])
ylim([-40,5])
set(gca,'FontSize',fs)

xlabel('k')
%ylabel('log(abs(a))')
%title('Fourier coefficients of solution on [0,2pi]')
temp=get(gcf,'Position'); set(gcf,'Position',temp+[100 0 0 0]);


set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperPosition', [0 0 10 10]);
% filename=[dirname,'/dispersion0'];
datei_a = strcat(datei,'_fourier');
datei_b = strcat(datei,'_re_im');
datei_c = strcat(datei,'_abs');
datei_aeps = strcat(datei_a,'.eps');
datei_beps = strcat(datei_b,'.eps');
datei_ceps = strcat(datei_c,'.eps');
%print(gcf,'-depsc','sol19_branch4_a')
%system(['epstopdf sol19_branch4_a.eps']);
%system(['rm sol19_branch4_a.eps']);
print(gcf,'-depsc',datei_a)
system(['epstopdf ' datei_aeps]);
%system(['rm ' datei_aeps]);
%close(gcf) 

figure(60)
lw=2; % Line Width
plot(xxvec,real(ahat),'b-', xxvec,imag(ahat),'b--','LineWidth',lw)
xlim([0, 2*pi])
set(gca,'FontSize',fs)
xlabel('x')

set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperPosition', [0 0 10 10]);
% filename=[dirname,'/dispersion0'];
%print(gcf,'-depsc','sol19_branch4_b')
%system(['epstopdf sol19_branch4_b.eps']);
%system(['rm sol19_branch4_b.eps']);
print(gcf,'-depsc',datei_b)
system(['epstopdf ' datei_beps]);
%system(['rm ' datei_beps]);

figure(70)
lw=2; % Line Width
plot(xxvec,abs(ahat),'b-','LineWidth',lw)
xlim([0, 2*pi])
set(gca,'FontSize',fs)
xlabel('x')

set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperPosition', [0 0 10 10]);
% filename=[dirname,'/dispersion0'];
%print(gcf,'-depsc','sol19_branch4_b')
%system(['epstopdf sol19_branch4_b.eps']);
%system(['rm sol19_branch4_b.eps']);
print(gcf,'-depsc',datei_c)
system(['epstopdf ' datei_ceps]);
%system(['rm ' datei_ceps]);

end
cd(path2)
exit
